part of tecfy_core_client_package;

class DropdownAdvancedEntity extends StatefulWidget {
  /// entity data if the [entityId] is null
  final Entity entity;

  /// entity dbName if the [entity] property is null
  final String entityId;

  /// url to fill the data
  final String? url;

  /// label string
  final String? label;

  /// label description
  final String? description;

  /// is required
  final bool isRequired;
  final bool readOnly;

  /// full model data
  final Map<String, dynamic> data;

  /// property
  final PropertyDropdown property;

  /// on changed
  final void Function(String?, Map<String, dynamic>?)? onChanged;
  DropdownAdvancedEntity(this.entityId,
      {required this.property,
      this.label,
      this.description,
      this.isRequired = false,
      this.readOnly = false,
      required this.entity,
      this.url,
      this.onChanged,
      required this.data});
  @override
  _DropdownAdvancedEntityState createState() => _DropdownAdvancedEntityState();
}

class _DropdownAdvancedEntityState extends AppState<DropdownAdvancedEntity> {
  List<dynamic>? _listData;

  String? get _value => _dataValue?['id'];
  String? _initValue;
  Map<String, dynamic>? _dataValue;
  Entity? _entity;
  String? _entityPropName;
  dynamic _defaultParentData = {'id': null};

  @override
  void initState() {
    super.initState();
    _init();
  }

  _init() {
    _initValue = widget.data[widget.property.dbName];

    if (_entity == null) _entity = widget.entity;
    if (_entityPropName == null)
      _entityPropName = widget.property.dbName
          .substring(0, widget.property.dbName.indexOf("Id"));

    if (_dataValue == null || _dataValue!['id'] != _initValue) {
      if (widget.data[_entityPropName] != null &&
          widget.data[_entityPropName]['id'] == _initValue)
        _dataValue = widget.data[_entityPropName];
    }

    if (_dataValue == null && _initValue != null && _initValue!.isNotEmpty) {
      // load data from server
      EntityService.getOne(widget.entity.dbName, _initValue).then((result) {
        if (result != null) {
          _dataValue = result;
          widget.data[_entityPropName!] = result;
          if (mounted) setState(() {});
        }
      });
    }
  }

  void _onItemSelectedFromList(BuildContext? context, Map<String, dynamic>? v) {
    try {
      _dataValue = v;
      widget.data[_entityPropName!] = _dataValue;
      if (widget.onChanged != null) widget.onChanged?.call(_value, _dataValue);
      setState(() {});
      context != null ? Navigator.pop(context) : print("null Navigator");
    } catch (err, trac) {
      print('xxxx error select from list $err');
      print(trac);
    }
  }

  _onSaveNewItem(BuildContext context, newDataRaw) async {
    try {
      var validateResult = EntityService.validateEntity(_entity, newDataRaw);
      if (!validateResult.success) {
        AppNavigator.showMessage(
            context, validateResult.message ?? 'Error', MessageType.error);
        return;
      }
      var addResult =
          await EntityService.saveEntity(_entity, newDataRaw, context, true);
      if (addResult.success) {
        _dataValue = addResult.data;
        widget.data[_entityPropName!] = _dataValue;
        _initValue = _dataValue!['id'];
        widget.onChanged?.call(_initValue!, _dataValue!);
        Navigator.pop(context, true); // close the add new item's form
      } else {
        AppNavigator.showMessage(
            context, addResult.message ?? 'Error', MessageType.error);
      }
    } catch (err, trace) {
      print('XXXXXXXXX erorr adding item');
      print(err);
      print(trace);
      AppNavigator.showMessage(context, 'Error Adding Item', MessageType.error);
    }
  }

  _openSearchDialog() async {
    await AppNavigator.showAppDialog(
        context,
        (listDialogContext) => Column(
              mainAxisAlignment: MainAxisAlignment.center,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: <Widget>[
                Expanded(child: Container()),
                Container(
                    width: min(500, MediaQuery.of(context).size.width),
                    height: min(
                        800,
                        MediaQuery.of(context).size.height -
                            MediaQuery.of(context).padding.top -
                            MediaQuery.of(context).padding.bottom -
                            50),
                    child: Card(
                        child: Column(
                            mainAxisAlignment: MainAxisAlignment.start,
                            crossAxisAlignment: CrossAxisAlignment.stretch,
                            children: <Widget>[
                          Container(
                              color: ThemeService.backgroundSecondColor,
                              child: Row(
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceBetween,
                                  children: <Widget>[
                                    Padding(
                                      padding: EdgeInsets.only(left: 10),
                                      child: getLableWidget(),
                                    ),
                                    IconButton(
                                        icon: Icon(Icons.close),
                                        onPressed: () =>
                                            Navigator.pop(listDialogContext))
                                  ])),
                          Expanded(
                              child: EntityListSearchComponent(
                            property: widget.property,
                            entity: widget.entity,
                            url: widget.url ?? '',
                            data: widget.data,
                            // isFilter: widget.isFilter ,
                            // filterValue: widget.filterValue,
                            // filterField: widget.filterField,
                            urlFilterData: widget.data,
                            onClick: (v) =>
                                _onItemSelectedFromList(listDialogContext, v),
                          )),
                          TextButton(
                            child: Text(trans('Add New Item'),
                                style: ThemeService.flatButtonTextStyle
                                    .copyWith(
                                        decoration: TextDecoration.underline)),
                            onPressed: () async {
                              Map<String, dynamic> newDataRaw =
                                  new Map<String, dynamic>();
                              var addNewResult =
                                  await AppNavigator.showAppDialogEx(
                                      listDialogContext,
                                      () => _getAddNewDialog(
                                          listDialogContext, newDataRaw),
                                      size: DialogBoxSizes.large);
                              if (addNewResult ?? false)
                                Navigator.pop(
                                    listDialogContext); // close the search form
                            },
                          )
                        ]))),
                Expanded(child: Container()),
              ],
            ));
    //loadData();
  }

  Widget _getAddNewDialog(addDialogContext, newDataRaw) {
    return Container(
      constraints: BoxConstraints(maxHeight: 500, maxWidth: 500),
      child: Center(
        child: Card(
          child: ListView(
            children: <Widget>[
              InLineForm(widget.entityId, true,
                  entity: _entity!,
                  inPopup: true,
                  data: newDataRaw,
                  onChanged: (v) {}),
              Padding(
                  padding: EdgeInsets.symmetric(horizontal: 20),
                  child: Row(
                    children: <Widget>[
                      Expanded(child: Container()),
                      Expanded(
                          child: AppButtonPost(
                              label: trans('Save'),
                              onPressed: () => _onSaveNewItem(
                                  addDialogContext, newDataRaw))),
                      SizedBox(width: 10),
                      Expanded(
                          child: Container(
                              height: 50.0,
                              child: TextButton(
                                  child: Text(trans('Cancel'),
                                      style: ThemeService.flatButtonTextStyle),
                                  onPressed: () =>
                                      Navigator.pop(addDialogContext, false)))),
                      Expanded(child: Container()),
                    ],
                  )),
            ],
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    if (_initValue != widget.data[widget.property.dbName] ||
        _dataValue == null ||
        _dataValue!['id'] != _initValue) _init();
    // print('----------- ${_entity!.displayField.dbName}');
    // if (_dataValue != null && _entity?.displayField.dbName != null)
    //   print("${_dataValue![_entity!.displayField.dbName]}");
    return _dataValue != null
        ? Stack(children: [
            MaterialButton(
                padding: EdgeInsets.all(0),
                onPressed: widget.readOnly ? null : _openSearchDialog,
                child: widget.label == null
                    ? Text(
                        _dataValue == null
                            ? trans('Click To Select')
                            : (_dataValue![_entity!.displayField.dbName] ??
                                'N/A'),
                        style: TextStyle(color: Colors.black, fontSize: 15))
                    : Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          SizedBox(height: 10),
                          getLableWidget(),
                          Text(
                              _dataValue == null
                                  ? trans('Click To Select')
                                  : (_dataValue![_entity!.displayField.dbName]
                                          ?.toString() ??
                                      'N/A'),
                              style:
                                  TextStyle(color: Colors.black, fontSize: 15)),
                          SizedBox(height: 10),
                          Container(
                              height: 1, color: ThemeService.textSubColor),
                          SizedBox(height: 5),
                        ],
                      )),
            widget.readOnly
                ? SizedBox()
                : Align(
                    alignment: Alignment.topLeft,
                    child: IconButton(
                        onPressed: () {
                          _onItemSelectedFromList(null, null);
                        },
                        icon: Icon(Icons.close, color: Colors.grey),
                        tooltip: trans("Clean")),
                  )
          ])
        : MaterialButton(
            padding: EdgeInsets.all(0),
            onPressed: _openSearchDialog,
            child: widget.label == null
                ? Text(
                    _dataValue == null
                        ? trans('Click To Select')
                        : (_dataValue![_entity!.displayField.dbName] ?? 'N/A'),
                    style: TextStyle(color: Colors.black, fontSize: 15))
                : Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      SizedBox(height: 10),
                      getLableWidget(),
                      Text(
                          _dataValue == null
                              ? trans('Click To Select')
                              : (_dataValue![_entity!.displayField.dbName]
                                      ?.toString() ??
                                  'N/A'),
                          style: TextStyle(color: Colors.black, fontSize: 15)),
                      SizedBox(height: 10),
                      Container(height: 1, color: ThemeService.textSubColor),
                      SizedBox(height: 5),
                    ],
                  ));
  }

  Widget getLableWidget() {
    return Row(
      children: [
        Tooltip(
            message: widget.description ?? '',
            child: Text(widget.label ?? '',
                style:
                    TextStyle(color: ThemeService.textSubColor, fontSize: 11))),
        widget.isRequired
            ? Text('*', style: TextStyle(color: Colors.red, fontSize: 11))
            : SizedBox(),
      ],
    );
  }
}
