part of tecfy_core_client_package;

class AppDataTable extends StatefulWidget {
  /// list of data
  final List<Map<String, dynamic>> data;

  /// column list of properties
  final List<Property> columns;

  /// if not null means will have select checkbox
  final String? isSelectedField;

  /// means user will not be able to change values
  final bool readOnly;

  /// on tap on row
  final Function(Map<String, dynamic>)? onRowTap;

  /// on select changed
  final Function(Map<String, dynamic>, bool)? onSelectedChanged;

  /// on cell tapped, send the row data along with the filed name
  final Function(Map<String, dynamic>, String)? onCellClicked;

  /// view data table with option to update value
  const AppDataTable(
      {Key? key,
      required this.columns,
      required this.data,
      this.onSelectedChanged,
      this.onCellClicked,
      this.onRowTap,
      this.readOnly = false,
      this.isSelectedField})
      : super(key: key);

  @override
  _AppDataTableState createState() => _AppDataTableState();
}

class _AppDataTableState extends State<AppDataTable> {
  bool _sortAscending = true;
  int? _sortColumnIndex;

  _onEditCell(Property propety, Map<String, dynamic> data) async {
    if (widget.onCellClicked != null) {
      widget.onCellClicked?.call(data, propety.dbName);
    }
    if (widget.readOnly || propety.readOnly) {
      widget.onRowTap?.call(data);
      return;
    }
    var result = await PopupPropertyComponent.show(context, propety, data);
    if (result != null) setState(() {});
  }

  _onSelectChanged(Map<String, dynamic> data, bool isSelected) {
    if (widget.isSelectedField == null) return;
    data[widget.isSelectedField!] = isSelected;
    widget.onSelectedChanged?.call(data, isSelected);
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Scrollbar(
      isAlwaysShown: true,
      showTrackOnHover: true,
      // controller: _controller.scrollController,
      child: SingleChildScrollView(
        padding: EdgeInsets.all(20),
        //controller: _scrollController,
        // scrollDirection: Axis.horizontal,
        child: DataTable(
          columnSpacing: 1,
          dividerThickness: 0.5,
          horizontalMargin: 1,
          checkboxHorizontalMargin: 5,
          showBottomBorder: false,
          sortAscending: _sortAscending,
          sortColumnIndex: _sortColumnIndex,
          showCheckboxColumn: widget.isSelectedField != null,
          columns: widget.columns
              .map<DataColumn>((h) => _getHeaderCell(h.nameAr,
                  numeric: h.valueType == ValueTypes.Number))
              .toList(),
          rows: widget.data
              .map((d) => DataRow(
                  selected: widget.isSelectedField == null
                      ? false
                      : d[widget.isSelectedField] ?? false,
                  onSelectChanged: (v) => _onSelectChanged(d, v ?? false),
                  cells: widget.columns
                      .map<DataCell>((h) => DataCell(
                            _getCell(h, d),
                            onTap: () => _onEditCell(h, d),
                            showEditIcon: !widget.readOnly && !h.readOnly,
                          ))
                      .toList()))
              .toList(),
        ),
      ),
    );
  }

  Widget _getCell(Property prop, data, {totals: false}) {
    if (prop.isImage) {
      if (data[prop.dbName + 'Id'] == null ||
          data[prop.dbName + 'Id'].toString().isEmpty) {
        return Container();
      }
      return Align(
          alignment: LocalConfigService.isArabic
              ? Alignment.centerRight
              : Alignment.centerLeft,
          child: Image(
              image: ImageService.imageProvider(data[prop.dbName + 'Id']) ??
                  TecfyCoreApp.logo!.image,
              height: 50));
    }

    var val = data[prop.dbName];
    if (totals) {
      if (val == null || val == 0 || val == '' || val == '0') {
        return Container();
      }
    } else {
      if (val == null) {
        return Container();
      }
    }

    Widget widgetValue = Padding(
      padding: EdgeInsets.only(
          top: prop.valueType == ValueTypes.Boolean ? 7 : 0, right: 2, left: 2),
      child: Align(
        alignment: LocalConfigService.isArabic
            ? Alignment.centerRight
            : Alignment.centerLeft,
        child: prop.getDisplayWidget(data),
      ),
    );
    return Padding(
        padding: EdgeInsets.only(
            top: 10.0,
            right: LocalConfigService.isArabic
                ? (((data['_tree_level'] ?? 0) as num).toDouble()) * 10.0
                : 0.0,
            left: LocalConfigService.isArabic
                ? 0.0
                : (((data['_tree_level'] ?? 0) as num).toDouble()) * 10.0),
        child: widgetValue);
  }

  DataColumn _getHeaderCell(String text,
      {bool isSelected = false, bool numeric = false}) {
    return DataColumn(
        onSort: (columnIndex, ascending) {
          setState(() {
            var p = widget.columns[columnIndex];
            widget.data.sort((a, b) => ascending
                ? a[p.dbName].compareTo(b[p.dbName])
                : b[p.dbName].compareTo(a[p.dbName]));
            _sortAscending = ascending;
            _sortColumnIndex = columnIndex;
          });
        },
        tooltip: text,
        numeric: numeric,
        label: Text('$text ${isSelected ? '*' : ''}',
            textAlign: TextAlign.start,
            style: TextStyle(
                fontSize: 12,
                fontWeight: isSelected ? FontWeight.bold : FontWeight.normal,
                color: ThemeService.secondryColor)));
  }
}
